/*
 * $Header: /export/CVS/UMT/libsrc/umtbase/misc.cc,v 1.3 1997/02/15 04:16:27 vlad Exp $
 * 
 *  $Log: misc.cc,v $
 *  Revision 1.3  1997/02/15 04:16:27  vlad
 *  License protection is optional now. Define LICENSE_PROTECTION macro to switch on license protection.
 *
 *  Revision 1.2  1996/10/22 21:41:20  lord
 *  Alpha version for SUN done and works.
 *  Still few warning to be eleminated and
 *  file locking to be done.
 *  file 'build_number' renamed to 'build_num'.
 *
 * 
 */

#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <dirent.h>
#include <unistd.h>
#include <sys/stat.h>
#include <pwd.h>
#include <netdb.h>
#include <ctype.h>

#include "UMT.h++"
#include "umtVersionNumber.h++"
#include "misc.h++"

#ifdef SUN
#include <sys/fcntl.h>
extern "C" { int  gethostname(char*, int); }
#endif

#define  MAX_BUF_SIZE    10240

int umtMAX(int a, int b) { return ( a>b ? a : b ); };
int umtMIN(int a, int b) { return ( a<b ? a : b ); };

char *ustrdup(const char *f)
{
    char *s;
    if(f)
	if((s=new char[strlen(f)+1])!=NULL)
	    return(strcpy(s,f));
    return NULL;
}

int filecopy( const char *source, const char *dist )
{
    int file_source, file_dist;
    char  buff[MAX_BUF_SIZE];
    int leng;
    

    if (( file_source = open( source, O_RDONLY )) == -1 )
 	return -1;
    if (( file_dist = open( dist, O_WRONLY | O_TRUNC | O_CREAT , 0644 )) == -1 )
    {
	close( file_source );
	return -1;
    }
    do 
    {
	if (( leng = read( file_source, buff , MAX_BUF_SIZE )) == -1 )
	    break;
	if (( leng = write( file_dist, buff , leng )) == -1 )
	    break;
    }
    while ( leng == MAX_BUF_SIZE );
    close( file_source );
    if ( leng == -1 )
    {
	unlink( dist );
	return -1;
    }
    else close( file_dist );
    return 0;
}

int fileappend( const char *source, const char *dist )
{
    int file_source, file_dist;
    char  buff[MAX_BUF_SIZE];
    int leng;
    
    if (( file_source = open( source, O_RDONLY )) == -1 )
 	return -1;
    if (( file_dist = open( dist, O_WRONLY | O_CREAT | O_APPEND , 0644 )) == -1 )
    {
	close( file_source );
	return -1;
    }
    do 
    {
	if (( leng = read( file_source, buff , MAX_BUF_SIZE )) == -1 )
	    break;
	if (( leng = write( file_dist, buff , leng )) == -1 )
	    break;
    }
    while ( leng == MAX_BUF_SIZE );
    close( file_source );
    if ( leng == -1 )
    {
	unlink( dist );
	return -1;
    }
    else close( file_dist );
    return 0;
}

int RemoveDirectory(const char *dir, int print)
{
    int ret = 0;
    char s[1024];
    struct stat buf;
    if (stat(dir,&buf)) return -100;
    if (!S_ISDIR(buf.st_mode)) return -1;

    if (print) fprintf(stderr,"Removing directory: <%s>\n",dir);

    DIR *dirp = opendir(dir);
    if (!dirp) return -101;
  
    struct dirent *dp;
    while ((dp=readdir(dirp)) != NULL) {
	sprintf(s,"%s/%s",dir,dp->d_name);
	if (strcmp(dp->d_name,".") && strcmp(dp->d_name,"..") ) {
	    if (stat(s,&buf)) return -102;
	    if (S_ISDIR(buf.st_mode)) {
		if (RemoveDirectory(s,print)) {
		    ret = 1;
		    fprintf(stderr,"Error removing directory: <%s>\n",s);
		}
	    }
	    else {
		if (unlink(s)==-1) {
		    ret = 2;
		    fprintf(stderr,"Error removing file: <%s>\n",s);
		}
		else if (print)
		    fprintf(stderr,"File: <%s> removed OK!\n",s);
	    }
	}
    }

    if (!ret && rmdir(dir)) {
	ret=3;
	fprintf(stderr,"Error removing directory: <%s>\n",dir);
    } 
    return ret;
} 

int MoveDirectory(const  char *source, const  char *dest,  int print)
{
    int ret = 0;
    char s1[1024], s2[1024];
    struct stat buf;
    struct dirent *dp;

    if (!rename(source, dest)) return 0;
    
    if (stat(source,&buf)) return -100;
    if (!S_ISDIR(buf.st_mode)) return -1;

    if (print) fprintf(stderr,"Moving directory: <%s>=><%s>\n",source,dest);

    DIR *dirp = opendir(source);
    if (!dirp) return -101;

    if (mkdir(dest,buf.st_mode)) {
	fprintf(stderr,"Error creating directory: <%s>\n",dest);	
	return -102;
    }
  
    while ((dp=readdir(dirp)) != NULL) {
	sprintf(s1,"%s/%s",source,dp->d_name);
	sprintf(s2,"%s/%s",dest,dp->d_name);
	if (strcmp(dp->d_name,".") && strcmp(dp->d_name,"..") ) {
	    if (stat(s1,&buf)) return -102;
	    if (S_ISDIR(buf.st_mode)) {
		if (MoveDirectory(s1,s2,print)) {
		    ret = 1;
		    fprintf(stderr,"Error moving directory: <%s>=><%s>\n",s1,s2);
		}
	    }
	    else {
		if (filecopy(s1,s2) || unlink(s1)==-1) {
		    ret = 2;
		    fprintf(stderr,"Error moving file: <%s>=><%s>\n",s1,s2);
		}
		else  if (print)
		    fprintf(stderr,"File: <%s>=><%s> moved OK!\n",s1,s2);
	    }
	}
    }

    if (!ret && rmdir(source)) {
	ret=3;
	fprintf(stderr,"Error moving directory: <%s>=><%s>\n",source,dest);
    } 
    return ret;
} 

int CopyDirectory(const  char *source, const  char *dest,  int print)
{
    int ret = 0;
    char s1[1024], s2[1024];
    struct stat buf;
    struct dirent *dp;

    if (stat(source,&buf)) return -100;
    if (!S_ISDIR(buf.st_mode)) return -1;

    if (print) fprintf(stderr,"Copying directory: <%s>=><%s>\n",source,dest);

    DIR *dirp = opendir(source);
    if (!dirp) return -101;

    if (mkdir(dest,buf.st_mode)) {
	fprintf(stderr,"Error creating directory: <%s>\n",dest);	
	return -102;
    }
  
    while ((dp=readdir(dirp)) != NULL) {
	sprintf(s1,"%s/%s",source,dp->d_name);
	sprintf(s2,"%s/%s",dest,dp->d_name);
	if (strcmp(dp->d_name,".") && strcmp(dp->d_name,"..") ) {
	    if (stat(s1,&buf)) return -102;
	    if (S_ISDIR(buf.st_mode)) {
		if (CopyDirectory(s1,s2,print)) {
		    ret = 1;
		    fprintf(stderr,"Error copying directory: <%s>=><%s>\n",s1,s2);
		}
	    }
	    else {
		if (filecopy(s1,s2)) {
		    ret = 2;
		    fprintf(stderr,"Error copying file: <%s>=><%s>\n",s1,s2);
		}
		else  if (print)
		    fprintf(stderr,"File: <%s>=><%s> copied OK!\n",s1,s2);
	    }
	}
    }

    return ret;
} 

long filesize(const char *file)
{
    FILE *fp = fopen(file,"r");
    if (!fp) return -1;
    fseek(fp,0L,SEEK_END);
    long size =  ftell(fp);
    fclose(fp);
    return size;
}


/* 
   Get user home directory 
   You must free pointer returned
*/ 

char *gethomedir()
{
    struct passwd *entry; 

    setpwent();
    entry=getpwuid(geteuid());
    endpwent();
    return  (char*)(entry?ustrdup(entry->pw_dir):nil);
}

char *getuserhomedir(const char *name)
{
    struct passwd *entry; 
    
    setpwent();
    entry=getpwnam(name);
    endpwent();
    return  (char*)(entry?ustrdup(entry->pw_dir):nil);
}


/*
  Get user internet' address
  You must free pointer returned
*/

char *get_internet_addr()
{
    char s[MAXSTRING], host_name[MAXSTRING];
    struct hostent *he;
    struct passwd *entry; 
    
    setpwent();
    entry=getpwuid(geteuid());
    endpwent();
    gethostname(host_name,1023);
    he = gethostbyname(host_name);
    if (!( entry && he )) return nil;
    sprintf(s,"%s@%s",entry->pw_name, he->h_name);
    return ustrdup(s);
}


/*
  Get user Real Name
  You must free pointer returned
*/

char *get_real_name()
{
    char   real_name[1024];
    struct passwd  *pas;
    char   *p;

    setpwent();
    pas=getpwuid(geteuid());
    endpwent();
    
    strcpy(real_name,pas->pw_gecos);
    if (*real_name != '&') {
	p = strchr(real_name,',');
	if ( p ) *p = '\0';
    }
    else {
	strcpy(real_name, pas->pw_name);
	*real_name = toupper(*real_name);
    }
    return ustrdup(real_name);
}


int IsSevenBitFile(const char *file)
{
    int i, n;
    unsigned  char buffer[MAX_BUF_SIZE+1];
    FILE *fp = fopen(file,"r");
    if (!fp) return ERROR_OPEN_FILE;
    
    while ((n=fread(buffer,1,MAX_BUF_SIZE,fp)) > 0)
	for (i=0; i<n; ++i)
	    if (buffer[i] > 127) {
		fclose(fp);
		return 0;
	    }

    fclose(fp);
    return 1;
}


int umt_flock( int fd, int operation )
{
#ifdef LINUX 
    return flock( fd, operation );
#else
    struct flock sfl;
                              
    int cmd = ( operation & LOCK_NB )? F_SETLK : F_SETLKW;
                              
    sfl.l_whence = 0;
    sfl.l_start = sfl.l_len = 0;
    sfl.l_pid = getpid();     
    switch ( operation & ~LOCK_NB )
    {
    case LOCK_EX :
	sfl.l_type = F_WRLCK;
	break;
    case LOCK_SH :  
	sfl.l_type = F_RDLCK;
	break;	
    case LOCK_UN :          
	sfl.l_type = F_UNLCK;
	break;
    default:  
	return -1;
    }
    return fcntl( fd, cmd, &sfl ); 
#endif //LINUX
    
}


umtSlist *CutString( const umtString& str,
		     int length,
		     const char* sep )
{
    umtSlist   *strings = NULL;
    umtString  cur_str, *str_ptr;
    int        cur_len, str_len = str.Length();
    char       *ptr, *next;

    if ( sep != NULL && str_len )
    {
	strings = new umtSlist;
	if ( !strings ) return NULL;
	for ( cur_len = 0; cur_len < str_len; cur_len += cur_str.Length())
	{
	    cur_str = umtString( str.Str() + cur_len, length );
	    if ( cur_str.Length() == length )
	    {
		for ( next = ptr = (char *)cur_str;
		      next;
		      next = strpbrk( ptr, sep )
		)
		    ptr = next + 1;
		if ( ptr != ( cur_str.Str() + 1 ))
		    cur_str.SetToLeft( ptr - cur_str.Str());
	    }
	    str_ptr = new umtString( cur_str );
	    if ( str_ptr == NULL )
	    {
		strings->Kill( NULL );
		Delete( strings );
		break;
	    }
	    strings->Append( str_ptr );	
	}
    }
    return strings;
}

umtString GenerateVersionString()
{
    umtString X_Mailer, us;
    
    X_Mailer ="Ultimate Mail Tool. version ";
    X_Mailer+=UMTVersion;
#ifdef LICENSE_PROTECTION
    if (UMTLicenceNumber) {
	X_Mailer+=" [license #";
	us.Sprintf("%ld",UMTLicenceNumber);
	X_Mailer+=us;
    }
    else 
	X_Mailer+=" [unregistered";
    X_Mailer+="]";
#endif
    return X_Mailer;
}




