/*
  $Id: umtLock.cc,v 1.3 1997/08/21 07:53:52 lord Exp $

  $Log: umtLock.cc,v $
  Revision 1.3  1997/08/21 07:53:52  lord
  Avoid parsing of utmp in Linux version. Using procfs instead. RPM spec for b4

  Revision 1.2  1997/08/20 02:01:59  lord
  Starting FreeBSD port

  Revision 1.1  1996/10/22 19:47:54  lord
  Due to stupidity of SUN CC, all .c++ files renamed to .cc.
  Some platform directories created in lib.
  Makefile.include for SUN created.

  Revision 1.1.1.1  1996/10/21 20:46:48  vlad
  Puting to CVS. Version 1.00B3

  Revision 2.0  1996/03/18  16:43:17  sebast
  Revision 2.0
  
  Revision 1.22  1996/03/18  15:17:13  vlad
  *** empty log message ***
  
  Revision 1.21  1996/01/31  16:25:11  vlad
  In critical error - exit(1).
  
  Revision 1.20  1995/11/22  11:46:44  vlad
  Change error type in  "umtLock: Second attempt to lock file fromr same process!!!"
  
  Revision 1.19  1995/11/11  14:34:14  vlad
  Nothing.
  
  Revision 1.18  1995/11/05  02:00:52  lord
  Funny bug in Lock fixed. I dunno why, but under Linux you should add O_RDWR!
  
  Revision 1.17  1995/11/04  08:41:03  lord
  Variable sys_start declared in class
  
  Revision 1.16  1995/11/04  08:37:22  lord
  time to sleep
  
  Revision 1.15  1995/11/04  07:34:34  lord
  Small bug fixed.
  
  Revision 1.14  1995/11/04  07:09:27  lord
  Use file descriptors and open(2) for locking.
  Now it will work, maybe.
  
  Revision 1.13  1995/11/04  07:02:42  lord
  Reimplement Lock() function. Now it is processing stale and ancient
  lock files correctly. Uses umtPid::IsExists,Read and Write.
  
  Revision 1.12  1995/10/28  11:54:33  vlad
  Debug massage added.
  
  Revision 1.11  1995/10/25  18:57:45  vlad
  Process id and system id added.
  
  Revision 1.10  1995/10/24  19:26:18  sebast
  ...
  
  Revision 1.9  1995/10/24  19:14:51  vlad
  PID included to lock file.
  
  Revision 1.8  1995/09/02  13:03:22  lord
  replace creat with open with O_EXCL flag. Works even on SGI.
  
  Revision 1.7  1995/08/24  13:19:09  lord
  String -> Str
  
  Revision 1.6  1995/08/22  13:10:01  lord
  LockMask->UMTLockMask
  
  Revision 1.5  1995/07/31  16:48:35  lord
  umt coding rules
  
  Revision 1.4  1995/07/27  17:24:49  lord
  sys/stat.h included
  
  Revision 1.3  1995/07/15  14:20:14  lord
  check for return value of creat are again made by compairson to -1.
  
  Revision 1.2  1995/07/15  14:12:35  lord
  check creat() return code for !=0 instead of ==-1
  
  Revision 1.1  1995/07/15  12:19:56  lord
  Initial revision
  
*/


#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <utmp.h>
#include <sys/stat.h>
#include "umtLock.h++"
#include "umtPid.h++"
#include "UMT.h++"

#ifdef FREEBSD
# include <sys/sysctl.h>
#endif

#ifdef LINUX
# define UPTIME_FILE  "/proc/uptime"
#endif

// -------------------------- class: umtLock  --------------------
// Created by lord@topaz.kiev.ua at Sat Jul 15 14:52:01 1995
// ----------------------------------------------------------------

const  mode_t  umtLock::UMTLockMask=(S_IRUSR|S_IWUSR);
       time_t  umtLock::sys_start = (time_t)-1;

umtLock::umtLock(umtPath p):umtMember(CLASS_UMT_LOCK)
{
    locked=0;
    path=p;
    if(sys_start==( time_t)-1) fill_sys_start();
}

umtLock::~umtLock()
{
    if(locked) UnLock();
}

umtPath umtLock::GetPath()
{
    return path;
}

status   umtLock::Lock()
{
    FILE    *ff;
    int     f;
    umtPid  pid((pid_t)0);
    time_t  lock_date;
    struct stat buf;

    if(locked) return ALREADY_LOCKED;
    f=-1;

    if((f=open(path.Str(),O_RDWR|O_CREAT|O_EXCL,UMTLockMask))==-1) // there is lock file. Can't crate
    {
	ff=fopen(path.Str(),"r");
	if(pid.Read(ff)!=OK)
	{
	    fclose(ff);
	    UMTError->Msg(umtError::Warning,"umtLock: Corrupted Lock file - Ignoring");
	}
	else
	{
	    fclose(ff);
	    stat(path.Str(),&buf);
	    lock_date=buf.st_ctime;
	    if(sys_start<lock_date) // Fresh lock
	    {
		if(pid.IsExists())
		{
		    UMTError->Msg( umtError::Critical,
				   pid.Pid==getpid()
				   ?
				   "umtLock: Second attempt to lock file fromr same process!!!"
				   :
				   "umtLock: File already locked by other process!"
		    );
//		    return NOT_LOCKED;
		}
		else // Locker is already dead. RIP.
		{
		    UMTError->Msg( umtError::Warning,"umtLock: Stale lock - ignoring.");
		}
	    }
	    else
	    {
		UMTError->Msg( umtError::Warning,"umtLock: Ancient lock - ignoring.");
	    }
	}
	unlink(path.Str());
	return Lock();
    }
    else // Locked!
    {
	pid.Pid=getpid();
	pid.Write(f);
	close(f);
	locked = 1;
	UMTError->Msg(umtError::Debug,"umtLock: Ok");
	return OK;
    }
}   


status   umtLock::UnLock()
{
    if(!locked) return NOT_LOCKED;
    if(path.Unlink()!=OK)
    {
	return ERROR_REMOVE;
    }
    else
    {
	locked=0;
	return OK;
    }
}

int      umtLock::IsLocked()
{
    return locked;
}


// Wow! This will take too long time. 
void umtLock::fill_sys_start( void )
{
    sys_start = 0;
    
#ifdef FREEBSD
    // following code stolen from 'w' command source from FreeBSD 2.2.2
    int mib[2];
    struct timeval  boottime;
    size_t size;
    /*
     * Print how long system has been up.
     * (Found by looking getting "boottime" from the kernel)
     */
    mib[0] = CTL_KERN;
    mib[1] = KERN_BOOTTIME;
    size = sizeof(boottime);
    if(sysctl(mib, 2, &boottime, &size, NULL, 0) != -1 &&
       boottime.tv_sec != 0)
	sys_start = boottime.tv_sec;
    
#else
#ifdef LINUX
    FILE *uf;

    if((uf=fopen(UPTIME_FILE,"r"))!=NULL)
    {
	double up, idle;
	
	if(fscanf(uf, "%lf %lf", &up, &idle) < 2)
	{
	    fclose(uf);
	    UMTError->Msg(umtError::Warning,"umtLock: Error reading /proc/uptime");
	    return;
	} else
	{
	    fclose(uf);
	    sys_start=time(nil)-int(up);
	    //fprintf(stderr,"UPTIME: %d\n",(int)sys_start);
	}
    }
#else
    struct utmp *entry;

    utmpname( WTMP_FILE );
    setutent();
    while (( entry = getutent()))
    {
	if ( entry->ut_type == BOOT_TIME )
	    sys_start = entry->ut_time;
    }
    endutent();
#endif //LINUX
#endif //FREEBSD
    
}






